@@ -24,6 +24,7 @@ void setupEnvironment()
     // ignore translations
     qputenv("XDG_DATA_DIRS", "does-not-exist");
 #endif
+    qputenv("TZ", "Asia/Kolkata");
 }
 Q_CONSTRUCTOR_FUNCTION(setupEnvironment)
 
@@ -121,12 +122,11 @@ void KFormatTest::formatValue()
              QString::fromUtf8("10.6 µm"));
 }
 
-enum TimeConstants {
-    MSecsInDay = 86400000,
-    MSecsInHour = 3600000,
-    MSecsInMinute = 60000,
-    MSecsInSecond = 1000,
-};
+constexpr quint64 MSecsInYear = (quint64)1000 * 60 * 60 * 24 * 365; // approximation of a year
+constexpr quint64 MSecsInDay = 1000 * 60 * 60 * 24;
+constexpr quint64 MSecsInHour = 3600000;
+constexpr quint64 MSecsInMinute = 60000;
+constexpr quint64 MSecsInSecond = 1000;
 
 void KFormatTest::formatDuration()
 {
@@ -291,10 +291,16 @@ void KFormatTest::formatAbbreviatedDuration()
     QCOMPARE(fmt.formatDuration(MSecsInSecond, opt), "1 sec"_L1);
     QCOMPARE(fmt.formatDuration(MSecsInMinute, opt), "1 min 0 sec"_L1);
     QCOMPARE(fmt.formatDuration(MSecsInMinute, opt | KFormat::HideSeconds), "1 min"_L1);
-    QCOMPARE(fmt.formatDuration(MSecsInHour, opt), "1 hr 0 min 0 sec"_L1);
+    QCOMPARE(fmt.formatDuration(MSecsInHour, opt), "1 hr 0 min"_L1);
     QCOMPARE(fmt.formatDuration(MSecsInHour, opt | KFormat::HideSeconds), "1 hr 0 min"_L1);
     QCOMPARE(fmt.formatDuration(MSecsInHour, opt | KFormat::HideSeconds | KFormat::FoldHours), "60 min"_L1);
     QCOMPARE(fmt.formatDuration(12 * MSecsInHour + 59 * MSecsInMinute, opt | KFormat::HideSeconds), "12 hr 59 min"_L1);
+    QCOMPARE(fmt.formatDuration(17 * MSecsInDay + 2 * MSecsInHour + 19 * MSecsInMinute, opt), "17 d 2 hr"_L1);
+    QCOMPARE(fmt.formatDuration(19 * MSecsInMinute + 5 * MSecsInSecond, opt), "19 min 5 sec"_L1);
+    QCOMPARE(fmt.formatDuration(19 * MSecsInMinute + 5 * MSecsInSecond, opt | KFormat::HideSeconds), "19 min"_L1);
+    QCOMPARE(fmt.formatDuration(2 * MSecsInYear + 2 * MSecsInHour + 19 * MSecsInMinute, opt), "2 yr"_L1);
+
+    QCOMPARE(fmt.formatDuration(2 * MSecsInYear + 4 * MSecsInDay + 19 * MSecsInMinute, opt), "2 yr 4 d"_L1);
 }
 
 void KFormatTest::formatDecimalDuration()
@@ -395,10 +401,18 @@ void KFormatTest::formatRelativeDate()
     testDateTime = now.addSecs(-1);
     QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::ShortFormat), QStringLiteral("Just now"));
 
+    // 1 second ahead
+    testDateTime = now.addSecs(1);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::ShortFormat), QStringLiteral("Now"));
+
     // 5 minutes ago
     testDateTime = now.addSecs(-300);
     QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::ShortFormat), QStringLiteral("5 minute(s) ago"));
 
+    // 5 minutes ahead
+    testDateTime = now.addSecs(300);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::ShortFormat), "In 5 minute(s)"_L1);
+
     testDateTime = QDateTime(QDate::currentDate().addDays(8), QTime(3, 0, 0));
     QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::LongFormat),
              QStringLiteral("%1 at %2")
@@ -417,6 +431,54 @@ void KFormatTest::formatRelativeDate()
     QLocale frenchLocal = QLocale::French;
     KFormat formatFrench(frenchLocal);
     QCOMPARE(formatFrench.formatRelativeDateTime(testDateTime, QLocale::LongFormat), QStringLiteral("Dimanche 3 octobre 2021 at 05:33"));
+
+    // narrow format
+    now = QDateTime::currentDateTime();
+    testDateTime = now.addSecs(-1);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::NarrowFormat), "Just now"_L1);
+    testDateTime = now.addSecs(1);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::NarrowFormat), "Now"_L1);
+
+    testDateTime = QDateTime::currentDateTime().addSecs(-330);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::NarrowFormat), "5 min(s) ago"_L1);
+    testDateTime = QDateTime::currentDateTime().addSecs(310);
+    QCOMPARE(format.formatRelativeDateTime(testDateTime, QLocale::NarrowFormat), "5 min(s)"_L1);
+}
+
+void KFormatTest::formatTime()
+{
+    if (QTimeZone::systemTimeZone().id() != "Asia/Kolkata") {
+        qDebug() << QTimeZone::systemTimeZone().id();
+        QSKIP("Changing timezone not working on this platform!");
+    }
+
+    KFormat fmt(QLocale(u"en_GB"));
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone::LocalTime}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviation), "12:23"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone::utc()}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviation), "12:23 UTC"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone::utc()}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviationIfNeeded), "12:23 UTC"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone("Asia/Kolkata")}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone("Asia/Kolkata")}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviation), "12:23 IST"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone("Europe/Brussels")}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23 CEST"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 1, 23}, {12, 23, 45}, QTimeZone("Europe/Brussels")}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviation),
+             "12:23 CET"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 1, 23}, {12, 23, 45}, QTimeZone("Europe/Brussels")}, QLocale::ShortFormat, KFormat::DoNotAddTimeZone), "12:23"_L1);
+
+    QCOMPARE(fmt.formatTime({{2025, 1, 23}, {12, 23, 45}, QTimeZone::fromSecondsAheadOfUtc(0)}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23 UTC"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 1, 23}, {12, 23, 45}, QTimeZone::fromSecondsAheadOfUtc(8 * 3600)},
+                            QLocale::ShortFormat,
+                            KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23 UTC+08:00"_L1);
+    QCOMPARE(fmt.formatTime({{2025, 1, 23}, {12, 23, 45}, QTimeZone::fromSecondsAheadOfUtc(5.5 * 3600)},
+                            QLocale::ShortFormat,
+                            KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23"_L1);
+
+    KFormat de_fmt(QLocale(u"de_DE"));
+    QCOMPARE(de_fmt.formatTime({{2025, 5, 23}, {12, 23, 45}, QTimeZone("Europe/Brussels")}, QLocale::ShortFormat, KFormat::AddTimezoneAbbreviationIfNeeded),
+             "12:23 MESZ"_L1);
 }
 
 void KFormatTest::formatDistance()
